""" A ScheduleRunCommand Command """
from cleo import Command
from scheduler.Task import Task
import pendulum


class ScheduleRunCommand(Command):
    """
    Run the scheduled tasks

    schedule:run
        {--t|task=None : Name of task you want to run}
    """

    def handle(self):
        from wsgi import container as app
        tasks = app.collect(Task)
        
        for task_key, task_class in tasks.items():
            # Resolve the task with the container
            if self.option('task') != 'None':
                if self.option('task') != task_key and self.option('task') != task_class.name:
                    continue

            task = app.resolve(task_class)

            # If the class should run then run it
            if task.should_run():
                task.handle()



''' A ScheduleProvider Service Provider '''
from masonite.provider import ServiceProvider
from ..commands.ScheduleRunCommand import ScheduleRunCommand
from ..commands.CreateTaskCommand import CreateTaskCommand

class ScheduleProvider(ServiceProvider):

    def register(self):
        self.app.bind('ScheduleCommand', ScheduleRunCommand())
        self.app.bind('CreateTaskCommand', CreateTaskCommand())

    def boot(self):
        pass



""" A CreateTask Command """
from cleo import Command
import os


class CreateTaskCommand(Command):
    """
    Create a new task

    task
        {name : Name of the task you want to create}
    """

    def handle(self):
        task = self.argument('name')
        if not os.path.isfile('app/tasks/{0}.py'.format(task)):
            if not os.path.exists(os.path.dirname('app/tasks/{0}.py'.format(task))):
                # Create the path to the Task if it does not exist
                os.makedirs(os.path.dirname('app/tasks/{0}.py'.format(task)))

            f = open('app/tasks/{0}.py'.format(task), 'w+')

            f.write("''' Task Module Description '''\n".format(task))
            f.write('from scheduler.Task import Task\n\n')
            f.write("class {0}(Task):\n    ''' Task description '''\n\n    ".format(task))
            f.write("def __init__(self):\n        pass\n\n    ".format(task))
            f.write("def handle(self):\n        pass\n".format(task))

            self.info('Task Created Successfully!')
        else:
            self.error('Task Already Exists!')


import pendulum

class Task:

    run_every = False
    run_at = False
    run_every_hour = False
    run_every_minute = False
    twice_daily = False
    run_weekly = False

    _date = None

    name = ''

    def __init__(self):
        """
        Should only be on the child class. Also needs to be resolved by the container.
        """

        pass


    def handle(self):
        """
        Fires the task
        """

        pass


    def log(self):
        """
        Log into scheduler cache
        """
        pass


    def should_run(self, date=None):
        """
        If the task should run
        """

        # set the date
        self._set_date()

        if self._verify_run():
            return True

        return False


    def _set_date(self):
        if not self._date:
            self._date = pendulum.now()
            if hasattr(self, 'timezone'):
                self._date.in_timezone(self.timezone)


    def _verify_run(self):
        if self.run_every:
            time = self.run_every.split(' ')

            if time[1] in ('minute', 'minutes'):
                time = int(time[0])
                if self._date.minute == 0 or self._date.minute % time == 0 or time == 1:
                    return True

            elif time[1] in ('hour', 'hours'):
                time = int(time[0])
                if self._date.hour % time == 0 and self._date.minute == 0:
                    return True

            elif time[1] in ('day', 'days'):
                time = int(time[0])
                if self._date.day_of_year % time == 0 and (self._date.hour == 0 and self._date.minute == 0 or self._verify_run_at()):
                    return True
            elif time[1] in ('month', 'months'):
                time = int(time[0])
                if self._date.month % time == 0 and self._date.day == 1 and (self._date.hour == 0 and self._date.minute == 0 or (self._date.day == 0 and self._verify_run_at())):
                    return True

        elif self.run_at:
            return self._verify_run_at()
                
        if self.run_every_minute:
            if self._date.minute / 1  == 1:
                return True
        elif self.run_every_hour:
            if self._date.hour / 1 == 1:
                return True  
        elif self.twice_daily:
            if self._date.hour in self.twice_daily:
                return True
 
        return False

    def _verify_run_every():
        pass
    
    def _verify_run_at(self):
        if self._date.minute < 10:
            minute = "0{}".format(self._date.minute)
        else:
            minute = self._date.minute

        if "{0}:{1}".format(self._date.hour, minute) == self.run_at:
            return True
        
        return False


from setuptools import setup

setup(
    name="masonite-scheduler",
    version='1.0.2',
    packages=[
        'scheduler',
        'scheduler.commands',
        'scheduler.providers'
    ],
    install_requires=[],
    include_package_data=True,
)
